@@ -38,6 +38,8 @@ static const struct debug_control aco_debug_options[] = {
    {"novn", DEBUG_NO_VN},
    {"noopt", DEBUG_NO_OPT},
    {"nosched", DEBUG_NO_SCHED},
+   {"perfinfo", DEBUG_PERF_INFO},
+   {"liveinfo", DEBUG_LIVE_INFO},
    {NULL, 0}
 };
 
@@ -60,7 +62,7 @@ void init()
 
 void init_program(Program *program, Stage stage, struct radv_shader_info *info,
                   enum chip_class chip_class, enum radeon_family family,
-                  ac_shader_config *config)
+                  bool wgp_mode, ac_shader_config *config)
 {
    program->stage = stage;
    program->config = config;
@@ -93,35 +95,71 @@ void init_program(Program *program, Stage stage, struct radv_shader_info *info,
    program->wave_size = info->wave_size;
    program->lane_mask = program->wave_size == 32 ? s1 : s2;
 
-   program->lds_alloc_granule = chip_class >= GFX7 ? 512 : 256;
-   program->lds_limit = chip_class >= GFX7 ? 65536 : 32768;
+   program->dev.lds_encoding_granule = chip_class >= GFX7 ? 512 : 256;
+   program->dev.lds_alloc_granule = chip_class >= GFX10_3 ? 1024 : program->dev.lds_encoding_granule;
+   program->dev.lds_limit = chip_class >= GFX7 ? 65536 : 32768;
    /* apparently gfx702 also has 16-bank LDS but I can't find a family for that */
-   program->has_16bank_lds = family == CHIP_KABINI || family == CHIP_STONEY;
+   program->dev.has_16bank_lds = family == CHIP_KABINI || family == CHIP_STONEY;
 
-   program->vgpr_limit = 256;
-   program->vgpr_alloc_granule = 3;
+   program->dev.vgpr_limit = 256;
+   program->dev.physical_vgprs = 256;
+   program->dev.vgpr_alloc_granule = 4;
 
    if (chip_class >= GFX10) {
-      program->physical_sgprs = 2560; /* doesn't matter as long as it's at least 128 * 20 */
-      program->sgpr_alloc_granule = 127;
-      program->sgpr_limit = 106;
+      program->dev.physical_sgprs = 5120; /* doesn't matter as long as it's at least 128 * 40 */
+      program->dev.physical_vgprs = program->wave_size == 32 ? 1024 : 512;
+      program->dev.sgpr_alloc_granule = 128;
+      program->dev.sgpr_limit = 108; /* includes VCC, which can be treated as s[106-107] on GFX10+ */
       if (chip_class >= GFX10_3)
-         program->vgpr_alloc_granule = program->wave_size == 32 ? 15 : 7;
+         program->dev.vgpr_alloc_granule = program->wave_size == 32 ? 16 : 8;
       else
-         program->vgpr_alloc_granule = program->wave_size == 32 ? 7 : 3;
+         program->dev.vgpr_alloc_granule = program->wave_size == 32 ? 8 : 4;
    } else if (program->chip_class >= GFX8) {
-      program->physical_sgprs = 800;
-      program->sgpr_alloc_granule = 15;
+      program->dev.physical_sgprs = 800;
+      program->dev.sgpr_alloc_granule = 16;
+      program->dev.sgpr_limit = 102;
       if (family == CHIP_TONGA || family == CHIP_ICELAND)
-         program->sgpr_limit = 94; /* workaround hardware bug */
-      else
-         program->sgpr_limit = 102;
+         program->dev.sgpr_alloc_granule = 96; /* workaround hardware bug */
    } else {
-      program->physical_sgprs = 512;
-      program->sgpr_alloc_granule = 7;
-      program->sgpr_limit = 104;
+      program->dev.physical_sgprs = 512;
+      program->dev.sgpr_alloc_granule = 8;
+      program->dev.sgpr_limit = 104;
    }
 
+   program->dev.max_wave64_per_simd = 10;
+   if (program->chip_class >= GFX10_3)
+      program->dev.max_wave64_per_simd = 16;
+   else if (program->chip_class == GFX10)
+      program->dev.max_wave64_per_simd = 20;
+   else if (program->family >= CHIP_POLARIS10 && program->family <= CHIP_VEGAM)
+      program->dev.max_wave64_per_simd = 8;
+
+   program->dev.simd_per_cu = program->chip_class >= GFX10 ? 2 : 4;
+
+   switch (program->family) {
+   /* GFX8 APUs */
+   case CHIP_CARRIZO:
+   case CHIP_STONEY:
+   /* GFX9 APUS */
+   case CHIP_RAVEN:
+   case CHIP_RAVEN2:
+   case CHIP_RENOIR:
+      program->dev.xnack_enabled = true;
+      break;
+   default:
+      break;
+   }
+
+   program->dev.sram_ecc_enabled = program->family == CHIP_ARCTURUS;
+   /* apparently gfx702 also has fast v_fma_f32 but I can't find a family for that */
+   program->dev.has_fast_fma32 = program->chip_class >= GFX9;
+   if (program->family == CHIP_TAHITI ||
+       program->family == CHIP_CARRIZO ||
+       program->family == CHIP_HAWAII)
+      program->dev.has_fast_fma32 = true;
+
+   program->wgp_mode = wgp_mode;
+
    program->next_fp_mode.preserve_signed_zero_inf_nan32 = false;
    program->next_fp_mode.preserve_signed_zero_inf_nan16_64 = false;
    program->next_fp_mode.must_flush_denorms32 = false;
@@ -138,19 +176,19 @@ memory_sync_info get_sync_info(const Instruction* instr)
 {
    switch (instr->format) {
    case Format::SMEM:
-      return static_cast<const SMEM_instruction*>(instr)->sync;
+      return instr->smem().sync;
    case Format::MUBUF:
-      return static_cast<const MUBUF_instruction*>(instr)->sync;
+      return instr->mubuf().sync;
    case Format::MIMG:
-      return static_cast<const MIMG_instruction*>(instr)->sync;
+      return instr->mimg().sync;
    case Format::MTBUF:
-      return static_cast<const MTBUF_instruction*>(instr)->sync;
+      return instr->mtbuf().sync;
    case Format::FLAT:
    case Format::GLOBAL:
    case Format::SCRATCH:
-      return static_cast<const FLAT_instruction*>(instr)->sync;
+      return instr->flatlike().sync;
    case Format::DS:
-      return static_cast<const DS_instruction*>(instr)->sync;
+      return instr->ds().sync;
    default:
       return memory_sync_info();
    }
@@ -168,12 +206,12 @@ bool can_use_SDWA(chip_class chip, const aco_ptr<Instruction>& instr)
       return true;
 
    if (instr->isVOP3()) {
-      VOP3A_instruction *vop3 = static_cast<VOP3A_instruction*>(instr.get());
+      VOP3_instruction& vop3 = instr->vop3();
       if (instr->format == Format::VOP3)
          return false;
-      if (vop3->clamp && instr->format == asVOP3(Format::VOPC) && chip != GFX8)
+      if (vop3.clamp && instr->format == asVOP3(Format::VOPC) && chip != GFX8)
          return false;
-      if (vop3->omod && chip < GFX9)
+      if (vop3.omod && chip < GFX9)
          return false;
 
       //TODO: return true if we know we will use vcc
@@ -204,7 +242,7 @@ bool can_use_SDWA(chip_class chip, const aco_ptr<Instruction>& instr)
       return false;
 
    //TODO: return true if we know we will use vcc
-   if ((unsigned)instr->format & (unsigned)Format::VOPC)
+   if (instr->isVOPC())
       return false;
    if (instr->operands.size() >= 3 && !is_mac)
       return false;
@@ -230,14 +268,14 @@ aco_ptr<Instruction> convert_to_SDWA(chip_class chip, aco_ptr<Instruction>& inst
    std::copy(tmp->operands.cbegin(), tmp->operands.cend(), instr->operands.begin());
    std::copy(tmp->definitions.cbegin(), tmp->definitions.cend(), instr->definitions.begin());
 
-   SDWA_instruction *sdwa = static_cast<SDWA_instruction*>(instr.get());
+   SDWA_instruction& sdwa = instr->sdwa();
 
    if (tmp->isVOP3()) {
-      VOP3A_instruction *vop3 = static_cast<VOP3A_instruction*>(tmp.get());
-      memcpy(sdwa->neg, vop3->neg, sizeof(sdwa->neg));
-      memcpy(sdwa->abs, vop3->abs, sizeof(sdwa->abs));
-      sdwa->omod = vop3->omod;
-      sdwa->clamp = vop3->clamp;
+      VOP3_instruction& vop3 = tmp->vop3();
+      memcpy(sdwa.neg, vop3.neg, sizeof(sdwa.neg));
+      memcpy(sdwa.abs, vop3.abs, sizeof(sdwa.abs));
+      sdwa.omod = vop3.omod;
+      sdwa.clamp = vop3.clamp;
    }
 
    for (unsigned i = 0; i < instr->operands.size(); i++) {
@@ -247,27 +285,27 @@ aco_ptr<Instruction> convert_to_SDWA(chip_class chip, aco_ptr<Instruction>& inst
 
       switch (instr->operands[i].bytes()) {
       case 1:
-         sdwa->sel[i] = sdwa_ubyte;
+         sdwa.sel[i] = sdwa_ubyte;
          break;
       case 2:
-         sdwa->sel[i] = sdwa_uword;
+         sdwa.sel[i] = sdwa_uword;
          break;
       case 4:
-         sdwa->sel[i] = sdwa_udword;
+         sdwa.sel[i] = sdwa_udword;
          break;
       }
    }
    switch (instr->definitions[0].bytes()) {
    case 1:
-      sdwa->dst_sel = sdwa_ubyte;
-      sdwa->dst_preserve = true;
+      sdwa.dst_sel = sdwa_ubyte;
+      sdwa.dst_preserve = true;
       break;
    case 2:
-      sdwa->dst_sel = sdwa_uword;
-      sdwa->dst_preserve = true;
+      sdwa.dst_sel = sdwa_uword;
+      sdwa.dst_preserve = true;
       break;
    case 4:
-      sdwa->dst_sel = sdwa_udword;
+      sdwa.dst_sel = sdwa_udword;
       break;
    }
 
@@ -409,12 +447,12 @@ uint32_t get_reduction_identity(ReduceOp op, unsigned idx)
 bool needs_exec_mask(const Instruction* instr) {
    if (instr->isSALU())
       return instr->reads_exec();
-   if (instr->format == Format::SMEM || instr->isSALU())
+   if (instr->isSMEM() || instr->isSALU())
       return false;
-   if (instr->format == Format::PSEUDO_BARRIER)
+   if (instr->isBarrier())
       return false;
 
-   if (instr->format == Format::PSEUDO) {
+   if (instr->isPseudo()) {
       switch (instr->opcode) {
       case aco_opcode::p_create_vector:
       case aco_opcode::p_extract_vector:
@@ -441,4 +479,67 @@ bool needs_exec_mask(const Instruction* instr) {
    return true;
 }
 
+wait_imm::wait_imm() :
+   vm(unset_counter), exp(unset_counter), lgkm(unset_counter), vs(unset_counter) {}
+wait_imm::wait_imm(uint16_t vm_, uint16_t exp_, uint16_t lgkm_, uint16_t vs_) :
+      vm(vm_), exp(exp_), lgkm(lgkm_), vs(vs_) {}
+
+wait_imm::wait_imm(enum chip_class chip, uint16_t packed) : vs(unset_counter)
+{
+   vm = packed & 0xf;
+   if (chip >= GFX9)
+      vm |= (packed >> 10) & 0x30;
+
+   exp = (packed >> 4) & 0x7;
+
+   lgkm = (packed >> 8) & 0xf;
+   if (chip >= GFX10)
+      lgkm |= (packed >> 8) & 0x30;
+}
+
+uint16_t wait_imm::pack(enum chip_class chip) const
+{
+   uint16_t imm = 0;
+   assert(exp == unset_counter || exp <= 0x7);
+   switch (chip) {
+   case GFX10:
+   case GFX10_3:
+      assert(lgkm == unset_counter || lgkm <= 0x3f);
+      assert(vm == unset_counter || vm <= 0x3f);
+      imm = ((vm & 0x30) << 10) | ((lgkm & 0x3f) << 8) | ((exp & 0x7) << 4) | (vm & 0xf);
+      break;
+   case GFX9:
+      assert(lgkm == unset_counter || lgkm <= 0xf);
+      assert(vm == unset_counter || vm <= 0x3f);
+      imm = ((vm & 0x30) << 10) | ((lgkm & 0xf) << 8) | ((exp & 0x7) << 4) | (vm & 0xf);
+      break;
+   default:
+      assert(lgkm == unset_counter || lgkm <= 0xf);
+      assert(vm == unset_counter || vm <= 0xf);
+      imm = ((lgkm & 0xf) << 8) | ((exp & 0x7) << 4) | (vm & 0xf);
+      break;
+   }
+   if (chip < GFX9 && vm == wait_imm::unset_counter)
+      imm |= 0xc000; /* should have no effect on pre-GFX9 and now we won't have to worry about the architecture when interpreting the immediate */
+   if (chip < GFX10 && lgkm == wait_imm::unset_counter)
+      imm |= 0x3000; /* should have no effect on pre-GFX10 and now we won't have to worry about the architecture when interpreting the immediate */
+   return imm;
+}
+
+bool wait_imm::combine(const wait_imm& other)
+{
+   bool changed = other.vm < vm || other.exp < exp || other.lgkm < lgkm || other.vs < vs;
+   vm = std::min(vm, other.vm);
+   exp = std::min(exp, other.exp);
+   lgkm = std::min(lgkm, other.lgkm);
+   vs = std::min(vs, other.vs);
+   return changed;
+}
+
+bool wait_imm::empty() const
+{
+   return vm == unset_counter && exp == unset_counter &&
+          lgkm == unset_counter && vs == unset_counter;
+}
+
 }
