@@ -24,16 +24,17 @@
 #include "pipe/p_screen.h"
 
 #include "util/u_box.h"
-#include "util/u_format.h"
-#include "util/u_format_rgtc.h"
-#include "util/u_format_zs.h"
+#include "util/format/u_format.h"
+#include "util/format/u_format_rgtc.h"
+#include "util/format/u_format_zs.h"
 #include "util/u_inlines.h"
 #include "util/u_transfer_helper.h"
 
 
 struct u_transfer_helper {
    const struct u_transfer_vtbl *vtbl;
-   bool separate_z32s8;
+   bool separate_z32s8; /**< separate z32 and s8 */
+   bool separate_stencil; /**< separate stencil for all formats */
    bool fake_rgtc;
    bool msaa_map;
 };
@@ -88,11 +89,12 @@ u_transfer_helper_resource_create(struct pipe_screen *pscreen,
    enum pipe_format format = templ->format;
    struct pipe_resource *prsc;
 
-   if ((format == PIPE_FORMAT_Z32_FLOAT_S8X24_UINT) && helper->separate_z32s8) {
+   if ((helper->separate_stencil && util_format_is_depth_and_stencil(format)) ||
+       (format == PIPE_FORMAT_Z32_FLOAT_S8X24_UINT && helper->separate_z32s8)) {
       struct pipe_resource t = *templ;
       struct pipe_resource *stencil;
 
-      t.format = PIPE_FORMAT_Z32_FLOAT;
+      t.format = util_format_get_depth_only(format);
 
       prsc = helper->vtbl->resource_create(pscreen, &t);
       if (!prsc)
@@ -146,8 +148,8 @@ u_transfer_helper_resource_destroy(struct pipe_screen *pscreen,
 
 static bool needs_pack(unsigned usage)
 {
-   return (usage & PIPE_TRANSFER_READ) &&
-      !(usage & (PIPE_TRANSFER_DISCARD_WHOLE_RESOURCE | PIPE_TRANSFER_DISCARD_RANGE));
+   return (usage & PIPE_MAP_READ) &&
+      !(usage & (PIPE_MAP_DISCARD_WHOLE_RESOURCE | PIPE_MAP_DISCARD_RANGE));
 }
 
 /* In the case of transfer_map of a multi-sample resource, call back into
@@ -266,49 +268,65 @@ u_transfer_helper_transfer_map(struct pipe_context *pctx,
    if (!trans->ptr)
       goto fail;
 
-   if (prsc->format == PIPE_FORMAT_Z32_FLOAT_S8X24_UINT) {
+   if (util_format_is_depth_and_stencil(prsc->format)) {
       struct pipe_resource *stencil = helper->vtbl->get_stencil(prsc);
       trans->ptr2 = helper->vtbl->transfer_map(pctx, stencil, level,
                                                usage, box, &trans->trans2);
 
       if (needs_pack(usage)) {
-         util_format_z32_float_s8x24_uint_pack_z_float(trans->staging,
-                                                       ptrans->stride,
-                                                       trans->ptr,
-                                                       trans->trans->stride,
-                                                       width, height);
-         util_format_z32_float_s8x24_uint_pack_s_8uint(trans->staging,
-                                                       ptrans->stride,
-                                                       trans->ptr2,
-                                                       trans->trans2->stride,
-                                                       width, height);
+         switch (prsc->format) {
+         case PIPE_FORMAT_Z32_FLOAT_S8X24_UINT:
+            util_format_z32_float_s8x24_uint_pack_z_float(trans->staging,
+                                                          ptrans->stride,
+                                                          trans->ptr,
+                                                          trans->trans->stride,
+                                                          width, height);
+            util_format_z32_float_s8x24_uint_pack_s_8uint(trans->staging,
+                                                          ptrans->stride,
+                                                          trans->ptr2,
+                                                          trans->trans2->stride,
+                                                          width, height);
+            break;
+         case PIPE_FORMAT_Z24_UNORM_S8_UINT:
+            util_format_z24_unorm_s8_uint_pack_separate(trans->staging,
+                                                        ptrans->stride,
+                                                        trans->ptr,
+                                                        trans->trans->stride,
+                                                        trans->ptr2,
+                                                        trans->trans2->stride,
+                                                        width, height);
+            break;
+         default:
+            unreachable("Unexpected format");
+         }
       }
-   } else if (needs_pack(usage) &&
-              util_format_description(prsc->format)->layout == UTIL_FORMAT_LAYOUT_RGTC) {
-      switch (prsc->format) {
-      case PIPE_FORMAT_RGTC1_UNORM:
-      case PIPE_FORMAT_RGTC1_SNORM:
-      case PIPE_FORMAT_LATC1_UNORM:
-      case PIPE_FORMAT_LATC1_SNORM:
-         util_format_rgtc1_unorm_pack_rgba_8unorm(trans->staging,
-                                                  ptrans->stride,
-                                                  trans->ptr,
-                                                  trans->trans->stride,
-                                                  width, height);
-         break;
-      case PIPE_FORMAT_RGTC2_UNORM:
-      case PIPE_FORMAT_RGTC2_SNORM:
-      case PIPE_FORMAT_LATC2_UNORM:
-      case PIPE_FORMAT_LATC2_SNORM:
-         util_format_rgtc2_unorm_pack_rgba_8unorm(trans->staging,
-                                                  ptrans->stride,
-                                                  trans->ptr,
-                                                  trans->trans->stride,
-                                                  width, height);
-         break;
-      default:
-         assert(!"Unexpected format");
-         break;
+   } else if (util_format_description(prsc->format)->layout == UTIL_FORMAT_LAYOUT_RGTC) {
+      if (needs_pack(usage)) {
+         switch (prsc->format) {
+         case PIPE_FORMAT_RGTC1_UNORM:
+         case PIPE_FORMAT_RGTC1_SNORM:
+         case PIPE_FORMAT_LATC1_UNORM:
+         case PIPE_FORMAT_LATC1_SNORM:
+            util_format_rgtc1_unorm_pack_rgba_8unorm(trans->staging,
+                                                     ptrans->stride,
+                                                     trans->ptr,
+                                                     trans->trans->stride,
+                                                     width, height);
+            break;
+         case PIPE_FORMAT_RGTC2_UNORM:
+         case PIPE_FORMAT_RGTC2_SNORM:
+         case PIPE_FORMAT_LATC2_UNORM:
+         case PIPE_FORMAT_LATC2_SNORM:
+            util_format_rgtc2_unorm_pack_rgba_8unorm(trans->staging,
+                                                     ptrans->stride,
+                                                     trans->ptr,
+                                                     trans->trans->stride,
+                                                     width, height);
+            break;
+         default:
+            assert(!"Unexpected format");
+            break;
+         }
       }
    } else {
       unreachable("bleh");
@@ -333,13 +351,14 @@ flush_region(struct pipe_context *pctx, struct pipe_transfer *ptrans,
              const struct pipe_box *box)
 {
    struct u_transfer_helper *helper = pctx->screen->transfer_helper;
-   struct u_transfer *trans = u_transfer(ptrans);
+   /* using the function here hits an assert for the deinterleave cases */
+   struct u_transfer *trans = (struct u_transfer *)ptrans;
    enum pipe_format iformat, format = ptrans->resource->format;
    unsigned width = box->width;
    unsigned height = box->height;
    void *src, *dst;
 
-   if (!(ptrans->usage & PIPE_TRANSFER_WRITE))
+   if (!(ptrans->usage & PIPE_MAP_WRITE))
       return;
 
    if (trans->ss) {
@@ -383,7 +402,7 @@ flush_region(struct pipe_context *pctx, struct pipe_transfer *ptrans,
                                                       src,
                                                       ptrans->stride,
                                                       width, height);
-      /* fallthru */
+      FALLTHROUGH;
    case PIPE_FORMAT_X32_S8X24_UINT:
       dst = (uint8_t *)trans->ptr2 +
             (box->y * trans->trans2->stride) +
@@ -395,6 +414,22 @@ flush_region(struct pipe_context *pctx, struct pipe_transfer *ptrans,
                                                       ptrans->stride,
                                                       width, height);
       break;
+   case PIPE_FORMAT_Z24_UNORM_S8_UINT:
+      /* just do a strided 32-bit copy for depth; s8 can become garbage x8 */
+      util_format_z32_unorm_unpack_z_32unorm(dst, trans->trans->stride,
+                                             src, ptrans->stride,
+                                             width, height);
+      FALLTHROUGH;
+   case PIPE_FORMAT_X24S8_UINT:
+      dst = (uint8_t *)trans->ptr2 +
+            (box->y * trans->trans2->stride) +
+            (box->x * util_format_get_blocksize(PIPE_FORMAT_S8_UINT));
+
+      util_format_z24_unorm_s8_uint_unpack_s_8uint(dst, trans->trans2->stride,
+                                                   src, ptrans->stride,
+                                                   width, height);
+      break;
+
    case PIPE_FORMAT_RGTC1_UNORM:
    case PIPE_FORMAT_RGTC1_SNORM:
    case PIPE_FORMAT_LATC1_UNORM:
@@ -460,7 +495,7 @@ u_transfer_helper_transfer_unmap(struct pipe_context *pctx,
    if (handle_transfer(ptrans->resource)) {
       struct u_transfer *trans = u_transfer(ptrans);
 
-      if (!(ptrans->usage & PIPE_TRANSFER_FLUSH_EXPLICIT)) {
+      if (!(ptrans->usage & PIPE_MAP_FLUSH_EXPLICIT)) {
          struct pipe_box box;
          u_box_2d(0, 0, ptrans->box.width, ptrans->box.height, &box);
          flush_region(pctx, ptrans, &box);
@@ -478,6 +513,9 @@ u_transfer_helper_transfer_unmap(struct pipe_context *pctx,
             helper->vtbl->transfer_unmap(pctx, trans->trans2);
       }
 
+      pipe_resource_reference(&ptrans->resource, NULL);
+
+      free(trans->staging);
       free(trans);
    } else {
       helper->vtbl->transfer_unmap(pctx, ptrans);
@@ -487,6 +525,7 @@ u_transfer_helper_transfer_unmap(struct pipe_context *pctx,
 struct u_transfer_helper *
 u_transfer_helper_create(const struct u_transfer_vtbl *vtbl,
                          bool separate_z32s8,
+                         bool separate_stencil,
                          bool fake_rgtc,
                          bool msaa_map)
 {
@@ -494,6 +533,7 @@ u_transfer_helper_create(const struct u_transfer_vtbl *vtbl,
 
    helper->vtbl = vtbl;
    helper->separate_z32s8 = separate_z32s8;
+   helper->separate_stencil = separate_stencil;
    helper->fake_rgtc = fake_rgtc;
    helper->msaa_map = msaa_map;
 
@@ -505,3 +545,125 @@ u_transfer_helper_destroy(struct u_transfer_helper *helper)
 {
    free(helper);
 }
+
+
+/* these two functions 'deinterleave' are meant to be used without the corresponding
+ * resource_create/destroy hooks, as they perform the interleaving on-the-fly
+ *
+ * drivers should expect to be passed the same buffer repeatedly with the format changed
+ * to indicate which component is being mapped
+ */
+void *
+u_transfer_helper_deinterleave_transfer_map(struct pipe_context *pctx,
+                                            struct pipe_resource *prsc,
+                                            unsigned level, unsigned usage,
+                                            const struct pipe_box *box,
+                                            struct pipe_transfer **pptrans)
+{
+   struct u_transfer_helper *helper = pctx->screen->transfer_helper;
+   struct u_transfer *trans;
+   struct pipe_transfer *ptrans;
+   enum pipe_format format = prsc->format;
+   unsigned width = box->width;
+   unsigned height = box->height;
+
+   if (!((helper->separate_stencil && util_format_is_depth_and_stencil(format)) ||
+       (format == PIPE_FORMAT_Z32_FLOAT_S8X24_UINT && helper->separate_z32s8)))
+      return helper->vtbl->transfer_map(pctx, prsc, level, usage, box, pptrans);
+
+   debug_assert(box->depth == 1);
+
+   trans = calloc(1, sizeof(*trans));
+   if (!trans)
+      return NULL;
+
+   ptrans = &trans->base;
+   pipe_resource_reference(&ptrans->resource, prsc);
+   ptrans->level = level;
+   ptrans->usage = usage;
+   ptrans->box   = *box;
+   ptrans->stride = util_format_get_stride(format, box->width);
+   ptrans->layer_stride = ptrans->stride * box->height;
+
+   trans->staging = malloc(ptrans->layer_stride);
+   if (!trans->staging)
+      goto fail;
+
+   trans->ptr = helper->vtbl->transfer_map(pctx, prsc, level, usage | PIPE_MAP_DEPTH_ONLY, box,
+                                           &trans->trans);
+   if (!trans->ptr)
+      goto fail;
+
+   trans->ptr2 = helper->vtbl->transfer_map(pctx, prsc, level,
+                                            usage | PIPE_MAP_STENCIL_ONLY, box, &trans->trans2);
+   if (needs_pack(usage)) {
+      switch (prsc->format) {
+      case PIPE_FORMAT_Z32_FLOAT_S8X24_UINT:
+         util_format_z32_float_s8x24_uint_pack_z_float(trans->staging,
+                                                       ptrans->stride,
+                                                       trans->ptr,
+                                                       trans->trans->stride,
+                                                       width, height);
+         util_format_z32_float_s8x24_uint_pack_s_8uint(trans->staging,
+                                                       ptrans->stride,
+                                                       trans->ptr2,
+                                                       trans->trans2->stride,
+                                                       width, height);
+         break;
+      case PIPE_FORMAT_Z24_UNORM_S8_UINT:
+         util_format_z24_unorm_s8_uint_pack_separate(trans->staging,
+                                                     ptrans->stride,
+                                                     trans->ptr,
+                                                     trans->trans->stride,
+                                                     trans->ptr2,
+                                                     trans->trans2->stride,
+                                                     width, height);
+         break;
+      default:
+         unreachable("Unexpected format");
+      }
+   }
+
+   *pptrans = ptrans;
+   return trans->staging;
+
+fail:
+   if (trans->trans)
+      helper->vtbl->transfer_unmap(pctx, trans->trans);
+   if (trans->trans2)
+      helper->vtbl->transfer_unmap(pctx, trans->trans2);
+   pipe_resource_reference(&ptrans->resource, NULL);
+   free(trans->staging);
+   free(trans);
+   return NULL;
+}
+
+void
+u_transfer_helper_deinterleave_transfer_unmap(struct pipe_context *pctx,
+                                              struct pipe_transfer *ptrans)
+{
+   struct u_transfer_helper *helper = pctx->screen->transfer_helper;
+   enum pipe_format format = ptrans->resource->format;
+
+   if ((helper->separate_stencil && util_format_is_depth_and_stencil(format)) ||
+       (format == PIPE_FORMAT_Z32_FLOAT_S8X24_UINT && helper->separate_z32s8)) {
+      struct u_transfer *trans = (struct u_transfer *)ptrans;
+
+      if (!(ptrans->usage & PIPE_MAP_FLUSH_EXPLICIT)) {
+         struct pipe_box box;
+         u_box_2d(0, 0, ptrans->box.width, ptrans->box.height, &box);
+         flush_region(pctx, ptrans, &box);
+      }
+
+      helper->vtbl->transfer_unmap(pctx, trans->trans);
+      if (trans->trans2)
+         helper->vtbl->transfer_unmap(pctx, trans->trans2);
+
+      pipe_resource_reference(&ptrans->resource, NULL);
+
+      free(trans->staging);
+      free(trans);
+   } else {
+      helper->vtbl->transfer_unmap(pctx, ptrans);
+   }
+}
