@@ -313,12 +313,12 @@ i915_get_paramf(struct pipe_screen *screen, enum pipe_capf cap)
 {
    switch(cap) {
    case PIPE_CAPF_MAX_LINE_WIDTH:
-      /* fall-through */
+      FALLTHROUGH;
    case PIPE_CAPF_MAX_LINE_WIDTH_AA:
       return 7.5;
 
    case PIPE_CAPF_MAX_POINT_WIDTH:
-      /* fall-through */
+      FALLTHROUGH;
    case PIPE_CAPF_MAX_POINT_WIDTH_AA:
       return 255.0;
 
@@ -329,9 +329,9 @@ i915_get_paramf(struct pipe_screen *screen, enum pipe_capf cap)
       return 16.0;
 
    case PIPE_CAPF_MIN_CONSERVATIVE_RASTER_DILATE:
-      /* fall-through */
+      FALLTHROUGH;
    case PIPE_CAPF_MAX_CONSERVATIVE_RASTER_DILATE:
-      /* fall-through */
+      FALLTHROUGH;
    case PIPE_CAPF_CONSERVATIVE_RASTER_DILATE_GRANULARITY:
       return 0.0f;
 
