@@ -28,9 +28,10 @@
 #include "util/set.h"
 #include "util/slab.h"
 #include "util/u_debug.h"
+#include "util/u_threaded_context.h"
 #include "intel/blorp/blorp.h"
 #include "intel/dev/gen_debug.h"
-#include "intel/common/gen_l3_config.h"
+#include "intel/common/intel_l3_config.h"
 #include "intel/compiler/brw_compiler.h"
 #include "iris_batch.h"
 #include "iris_binder.h"
@@ -265,6 +266,17 @@ struct iris_cs_prog_key {
    struct iris_base_prog_key base;
 };
 
+union iris_any_prog_key {
+   struct iris_base_prog_key base;
+   struct iris_vue_prog_key vue;
+   struct iris_vs_prog_key vs;
+   struct iris_tcs_prog_key tcs;
+   struct iris_tes_prog_key tes;
+   struct iris_gs_prog_key gs;
+   struct iris_fs_prog_key fs;
+   struct iris_cs_prog_key cs;
+};
+
 /** @} */
 
 struct iris_depth_stencil_alpha_state;
@@ -389,6 +401,12 @@ struct iris_uncompiled_shader {
 
    /** Size (in bytes) of the local (shared) data passed as kernel inputs */
    unsigned kernel_shared_size;
+
+   /** List of iris_compiled_shader variants */
+   struct list_head variants;
+
+   /** Lock for the variants list */
+   simple_mtx_t lock;
 };
 
 enum iris_surface_group {
@@ -429,8 +447,14 @@ struct iris_binding_table {
  * (iris_uncompiled_shader), due to state-based recompiles (brw_*_prog_key).
  */
 struct iris_compiled_shader {
+   struct pipe_reference ref;
+
+   /** Link in the iris_uncompiled_shader::variants list */
    struct list_head link;
 
+   /** Key for this variant (but not for BLORP programs) */
+   union iris_any_prog_key key;
+
    /** Reference to the uploaded assembly. */
    struct iris_state_ref assembly;
 
@@ -514,8 +538,8 @@ struct iris_stream_output_target {
    /** Stride (bytes-per-vertex) during this transform feedback operation */
    uint16_t stride;
 
-   /** Has 3DSTATE_SO_BUFFER actually been emitted, zeroing the offsets? */
-   bool zeroed;
+   /** Does the next 3DSTATE_SO_BUFFER need to zero the offsets? */
+   bool zero_offset;
 };
 
 /**
@@ -539,6 +563,7 @@ struct iris_border_color_pool {
  */
 struct iris_context {
    struct pipe_context ctx;
+   struct threaded_context *thrctx;
 
    /** A debug callback for KHR_debug output. */
    struct pipe_debug_callback dbg;
@@ -552,6 +577,9 @@ struct iris_context {
    /** Slab allocator for iris_transfer_map objects. */
    struct slab_child_pool transfer_pool;
 
+   /** Slab allocator for threaded_context's iris_transfer_map objects */
+   struct slab_child_pool transfer_pool_unsync;
+
    struct blorp_context blorp;
 
    struct iris_batch batches[IRIS_BATCH_COUNT];
@@ -607,12 +635,18 @@ struct iris_context {
    struct {
       struct iris_uncompiled_shader *uncompiled[MESA_SHADER_STAGES];
       struct iris_compiled_shader *prog[MESA_SHADER_STAGES];
-      struct brw_vue_map *last_vue_map;
+      struct iris_compiled_shader *last_vue_shader;
+      struct {
+         unsigned size[4];
+         unsigned entries[4];
+         unsigned start[4];
+         bool constrained;
+      } urb;
 
-      /** List of shader variants whose deletion has been deferred for now */
-      struct list_head deleted_variants[MESA_SHADER_STAGES];
-
-      struct u_upload_mgr *uploader;
+      /** Uploader for shader assembly from the driver thread */
+      struct u_upload_mgr *uploader_driver;
+      /** Uploader for shader assembly from the threaded context */
+      struct u_upload_mgr *uploader_unsync;
       struct hash_table *cache;
 
       /** Is a GS or TES outputting points or lines? */
@@ -627,11 +661,6 @@ struct iris_context {
       struct iris_bo *scratch_bos[1 << 4][MESA_SHADER_STAGES];
    } shaders;
 
-   struct {
-      struct iris_query *query;
-      bool condition;
-   } condition;
-
    struct gen_perf_context *perf_ctx;
 
    /** Frame number for debug prints */
@@ -686,7 +715,10 @@ struct iris_context {
        */
       enum isl_aux_usage draw_aux_usage[BRW_MAX_DRAW_BUFFERS];
 
-      enum gen_urb_deref_block_size urb_deref_block_size;
+      /** Aux usage of the fb's depth buffer (which may or may not exist). */
+      enum isl_aux_usage hiz_usage;
+
+      enum intel_urb_deref_block_size urb_deref_block_size;
 
       /** Are depth writes enabled?  (Depth buffer may or may not exist.) */
       bool depth_writes_enabled;
@@ -822,7 +854,10 @@ void iris_copy_region(struct blorp_context *blorp,
 
 /* iris_draw.c */
 
-void iris_draw_vbo(struct pipe_context *ctx, const struct pipe_draw_info *info);
+void iris_draw_vbo(struct pipe_context *ctx, const struct pipe_draw_info *info,
+                   const struct pipe_draw_indirect_info *indirect,
+                   const struct pipe_draw_start_count *draws,
+                   unsigned num_draws);
 void iris_launch_grid(struct pipe_context *, const struct pipe_grid_info *);
 
 /* iris_pipe_control.c */
@@ -879,8 +914,9 @@ void iris_disk_cache_store(struct disk_cache *cache,
                            const void *prog_key,
                            uint32_t prog_key_size);
 struct iris_compiled_shader *
-iris_disk_cache_retrieve(struct iris_context *ice,
-                         const struct iris_uncompiled_shader *ish,
+iris_disk_cache_retrieve(struct iris_screen *screen,
+                         struct u_upload_mgr *uploader,
+                         struct iris_uncompiled_shader *ish,
                          const void *prog_key,
                          uint32_t prog_key_size);
 
@@ -888,12 +924,14 @@ iris_disk_cache_retrieve(struct iris_context *ice,
 
 void iris_init_program_cache(struct iris_context *ice);
 void iris_destroy_program_cache(struct iris_context *ice);
-void iris_print_program_cache(struct iris_context *ice);
 struct iris_compiled_shader *iris_find_cached_shader(struct iris_context *ice,
                                                      enum iris_program_cache_id,
                                                      uint32_t key_size,
                                                      const void *key);
-struct iris_compiled_shader *iris_upload_shader(struct iris_context *ice,
+struct iris_compiled_shader *iris_upload_shader(struct iris_screen *screen,
+                                                struct iris_uncompiled_shader *,
+                                                struct hash_table *driver_ht,
+                                                struct u_upload_mgr *uploader,
                                                 enum iris_program_cache_id,
                                                 uint32_t key_size,
                                                 const void *key,
@@ -905,11 +943,20 @@ struct iris_compiled_shader *iris_upload_shader(struct iris_context *ice,
                                                 unsigned kernel_input_size,
                                                 unsigned num_cbufs,
                                                 const struct iris_binding_table *bt);
-const void *iris_find_previous_compile(const struct iris_context *ice,
-                                       enum iris_program_cache_id cache_id,
-                                       unsigned program_string_id);
-void iris_delete_shader_variants(struct iris_context *ice,
-                                 struct iris_uncompiled_shader *ish);
+void iris_delete_shader_variant(struct iris_compiled_shader *shader);
+
+static inline void
+iris_shader_variant_reference(struct iris_compiled_shader **dst,
+                              struct iris_compiled_shader *src)
+{
+   struct iris_compiled_shader *old_dst = *dst;
+
+   if (pipe_reference(old_dst ? &old_dst->ref: NULL, src ? &src->ref : NULL))
+      iris_delete_shader_variant(old_dst);
+
+   *dst = src;
+}
+
 bool iris_blorp_lookup_shader(struct blorp_batch *blorp_batch,
                               const void *key,
                               uint32_t key_size,
@@ -946,7 +993,7 @@ int iris_get_driver_query_group_info(struct pipe_screen *pscreen,
                                      struct pipe_driver_query_group_info *info);
 
 /* iris_state.c */
-void gen9_toggle_preemption(struct iris_context *ice,
+void gfx9_toggle_preemption(struct iris_context *ice,
                             struct iris_batch *batch,
                             const struct pipe_draw_info *draw);
 
@@ -955,31 +1002,34 @@ void gen9_toggle_preemption(struct iris_context *ice,
 #ifdef genX
 #  include "iris_genx_protos.h"
 #else
-#  define genX(x) gen4_##x
+#  define genX(x) gfx4_##x
 #  include "iris_genx_protos.h"
 #  undef genX
-#  define genX(x) gen5_##x
+#  define genX(x) gfx5_##x
 #  include "iris_genx_protos.h"
 #  undef genX
-#  define genX(x) gen6_##x
+#  define genX(x) gfx6_##x
 #  include "iris_genx_protos.h"
 #  undef genX
-#  define genX(x) gen7_##x
+#  define genX(x) gfx7_##x
 #  include "iris_genx_protos.h"
 #  undef genX
-#  define genX(x) gen75_##x
+#  define genX(x) gfx75_##x
 #  include "iris_genx_protos.h"
 #  undef genX
-#  define genX(x) gen8_##x
+#  define genX(x) gfx8_##x
 #  include "iris_genx_protos.h"
 #  undef genX
-#  define genX(x) gen9_##x
+#  define genX(x) gfx9_##x
 #  include "iris_genx_protos.h"
 #  undef genX
-#  define genX(x) gen11_##x
+#  define genX(x) gfx11_##x
 #  include "iris_genx_protos.h"
 #  undef genX
-#  define genX(x) gen12_##x
+#  define genX(x) gfx12_##x
+#  include "iris_genx_protos.h"
+#  undef genX
+#  define genX(x) gfx125_##x
 #  include "iris_genx_protos.h"
 #  undef genX
 #endif
