@@ -138,7 +138,7 @@ static void ppir_regalloc_update_reglist_ssa(ppir_compiler *comp)
 {
    list_for_each_entry(ppir_block, block, &comp->block_list, list) {
       list_for_each_entry(ppir_node, node, &block->node_list, list) {
-         if (node->op == ppir_op_store_color)
+         if (node->is_end)
             continue;
 
          if (!node->instr || node->op == ppir_op_const)
@@ -151,6 +151,7 @@ static void ppir_regalloc_update_reglist_ssa(ppir_compiler *comp)
             if (dest->type == ppir_target_ssa) {
                reg = &dest->ssa;
                list_addtail(&reg->list, &comp->reg_list);
+               comp->reg_num++;
             }
          }
       }
@@ -270,7 +271,7 @@ static bool ppir_update_spilled_src(ppir_compiler *comp, ppir_block *block,
 
    ppir_load_node *load = ppir_node_to_load(load_node);
 
-   load->index = -comp->prog->stack_size; /* index sizes are negative */
+   load->index = -comp->prog->state.stack_size; /* index sizes are negative */
    load->num_components = num_components;
 
    ppir_dest *ld_dest = &load->dest;
@@ -310,12 +311,11 @@ static bool ppir_update_spilled_src(ppir_compiler *comp, ppir_block *block,
    ppir_dest *alu_dest = &move_alu->dest;
    alu_dest->type = ppir_target_ssa;
    alu_dest->ssa.num_components = num_components;
-   alu_dest->ssa.live_in = INT_MAX;
-   alu_dest->ssa.live_out = 0;
    alu_dest->ssa.spilled = true;
    alu_dest->write_mask = u_bit_consecutive(0, num_components);
 
    list_addtail(&alu_dest->ssa.list, &comp->reg_list);
+   comp->reg_num++;
 
    if (!ppir_instr_insert_node(load_node->instr, move_node))
       return false;
@@ -356,7 +356,7 @@ static bool ppir_update_spilled_dest_load(ppir_compiler *comp, ppir_block *block
 
    ppir_load_node *load = ppir_node_to_load(load_node);
 
-   load->index = -comp->prog->stack_size; /* index sizes are negative */
+   load->index = -comp->prog->state.stack_size; /* index sizes are negative */
    load->num_components = num_components;
 
    load->dest.type = ppir_target_pipeline;
@@ -416,11 +416,10 @@ static bool ppir_update_spilled_dest(ppir_compiler *comp, ppir_block *block,
 
    ppir_store_node *store = ppir_node_to_store(store_node);
 
-   store->index = -comp->prog->stack_size; /* index sizes are negative */
-   store->num_components = reg->num_components;
+   store->index = -comp->prog->state.stack_size; /* index sizes are negative */
 
-   store->src.type = dest->type;
-   store->src.reg = reg;
+   ppir_node_target_assign(&store->src, node);
+   store->num_components = reg->num_components;
 
    /* insert the new node as successor */
    ppir_node_foreach_succ_safe(node, dep) {
@@ -479,14 +478,14 @@ static bool ppir_regalloc_spill_reg(ppir_compiler *comp, ppir_reg *chosen)
 static ppir_reg *ppir_regalloc_choose_spill_node(ppir_compiler *comp,
                                                  struct ra_graph *g)
 {
-   float spill_costs[list_length(&comp->reg_list)];
+   float spill_costs[comp->reg_num];
    /* experimentally determined, it seems to be worth scaling cost of
     * regs in instructions that have used uniform/store_temp slots,
     * but not too much as to offset the num_components base cost. */
    const float slot_scale = 1.1f;
 
    list_for_each_entry(ppir_reg, reg, &comp->reg_list, list) {
-      if (reg->spilled || reg->live_out == INT_MAX) {
+      if (reg->spilled) {
          /* not considered for spilling */
          spill_costs[reg->regalloc_index] = 0.0f;
          continue;
@@ -535,7 +534,7 @@ static ppir_reg *ppir_regalloc_choose_spill_node(ppir_compiler *comp,
       }
    }
 
-   for (int i = 0; i < list_length(&comp->reg_list); i++)
+   for (int i = 0; i < comp->reg_num; i++)
       ra_set_node_spill_cost(g, i, spill_costs[i]);
 
    int r = ra_get_best_spill_node(g);
@@ -559,31 +558,40 @@ static ppir_reg *ppir_regalloc_choose_spill_node(ppir_compiler *comp,
 
 static void ppir_regalloc_reset_liveness_info(ppir_compiler *comp)
 {
-   int bitset_words = BITSET_WORDS(list_length(&comp->reg_list));
    int idx = 0;
 
    list_for_each_entry(ppir_reg, reg, &comp->reg_list, list) {
-      reg->live_in = INT_MAX;
-      reg->live_out = 0;
       reg->regalloc_index = idx++;
    }
 
    list_for_each_entry(ppir_block, block, &comp->block_list, list) {
-      if (block->def)
-         ralloc_free(block->def);
-      block->def = rzalloc_array(comp, BITSET_WORD, bitset_words);
+      list_for_each_entry(ppir_instr, instr, &block->instr_list, list) {
 
-      if (block->use)
-         ralloc_free(block->use);
-      block->use = rzalloc_array(comp, BITSET_WORD, bitset_words);
+         if (instr->live_mask)
+            ralloc_free(instr->live_mask);
+         instr->live_mask = rzalloc_array(comp, uint8_t,
+                                          reg_mask_size(comp->reg_num));
 
-      if (block->live_in)
-         ralloc_free(block->live_in);
-      block->live_in = rzalloc_array(comp, BITSET_WORD, bitset_words);
+         if (instr->live_set)
+            ralloc_free(instr->live_set);
+         instr->live_set = rzalloc_array(comp, BITSET_WORD, comp->reg_num);
 
-      if (block->live_out)
-         ralloc_free(block->live_out);
-      block->live_out = rzalloc_array(comp, BITSET_WORD, bitset_words);
+         if (instr->live_internal)
+            ralloc_free(instr->live_internal);
+         instr->live_internal = rzalloc_array(comp, BITSET_WORD, comp->reg_num);
+      }
+   }
+}
+
+static void ppir_all_interference(ppir_compiler *comp, struct ra_graph *g,
+                                  BITSET_WORD *liveness)
+{
+   int i, j;
+   BITSET_FOREACH_SET(i, liveness, comp->reg_num) {
+      BITSET_FOREACH_SET(j, liveness, comp->reg_num) {
+         ra_add_node_interference(g, i, j);
+      }
+      BITSET_CLEAR(liveness, i);
    }
 }
 
@@ -593,10 +601,8 @@ static bool ppir_regalloc_prog_try(ppir_compiler *comp, bool *spilled)
 {
    ppir_regalloc_reset_liveness_info(comp);
 
-   ppir_liveness_analysis(comp);
-
    struct ra_graph *g = ra_alloc_interference_graph(
-      comp->ra, list_length(&comp->reg_list));
+      comp->ra, comp->reg_num);
 
    int n = 0;
    list_for_each_entry(ppir_reg, reg, &comp->reg_list, list) {
@@ -606,32 +612,16 @@ static bool ppir_regalloc_prog_try(ppir_compiler *comp, bool *spilled)
       ra_set_node_class(g, n++, c);
    }
 
-   int n1 = 0;
-   list_for_each_entry(ppir_reg, reg1, &comp->reg_list, list) {
-      int n2 = n1 + 1;
-      list_for_each_entry_from(ppir_reg, reg2, reg1->list.next,
-                               &comp->reg_list, list) {
-         bool interference = false;
+   ppir_liveness_analysis(comp);
 
-         if (reg1->undef || reg2->undef)
-            interference = false;
-         else if (reg1->live_in < reg2->live_in) {
-            if (reg1->live_out > reg2->live_in)
-               interference = true;
+   list_for_each_entry(ppir_block, block, &comp->block_list, list) {
+      list_for_each_entry(ppir_instr, instr, &block->instr_list, list) {
+         int i;
+         BITSET_FOREACH_SET(i, instr->live_internal, comp->reg_num) {
+            BITSET_SET(instr->live_set, i);
          }
-         else if (reg1->live_in > reg2->live_in) {
-            if (reg2->live_out > reg1->live_in)
-               interference = true;
-         }
-         else
-            interference = true;
-
-         if (interference)
-            ra_add_node_interference(g, n1, n2);
-
-         n2++;
+         ppir_all_interference(comp, g, instr->live_set);
       }
-      n1++;
    }
 
    *spilled = false;
@@ -642,14 +632,14 @@ static bool ppir_regalloc_prog_try(ppir_compiler *comp, bool *spilled)
          /* stack_size will be used to assemble the frame reg in lima_draw.
           * It is also be used in the spilling code, as negative indices
           * starting from -1, to create stack addresses. */
-         comp->prog->stack_size++;
+         comp->prog->state.stack_size++;
          if (!ppir_regalloc_spill_reg(comp, chosen))
             goto err_out;
          /* Ask the outer loop to call back in. */
          *spilled = true;
 
          ppir_debug("spilled register %d/%d, num_components: %d\n",
-                    chosen->regalloc_index, list_length(&comp->reg_list),
+                    chosen->regalloc_index, comp->reg_num,
                     chosen->num_components);
          goto err_out;
       }
@@ -679,7 +669,7 @@ err_out:
 bool ppir_regalloc_prog(ppir_compiler *comp)
 {
    bool spilled = false;
-   comp->prog->stack_size = 0;
+   comp->prog->state.stack_size = 0;
 
    /* Set from an environment variable to force spilling
     * for debugging purposes, see lima_screen.c */
