@@ -124,7 +124,7 @@ bool ShaderFromNirProcessor::scan_instruction(nir_instr *instr)
       case nir_intrinsic_image_atomic_comp_swap:
       case nir_intrinsic_ssbo_atomic_comp_swap:
          m_sel.info.writes_memory = 1;
-         /* fallthrough */
+         FALLTHROUGH;
       case nir_intrinsic_image_load:
          m_ssbo_instr.set_require_rat_return_address();
          break;
@@ -168,6 +168,13 @@ static void remap_shader_info(r600_shader& sh_info,
                               std::vector<rename_reg_pair>& map,
                               UNUSED ValueMap& values)
 {
+   for (unsigned i = 0; i < sh_info.num_arrays; ++i) {
+      auto new_index = map[sh_info.arrays[i].gpr_start];
+      if (new_index.valid)
+         sh_info.arrays[i].gpr_start = new_index.new_reg;
+      map[sh_info.arrays[i].gpr_start].used = true;
+   }
+
    for (unsigned i = 0; i < sh_info.ninput; ++i) {
       sfn_log << SfnLog::merge << "Input " << i << " gpr:" << sh_info.input[i].gpr
               << " of map.size()\n";
@@ -274,8 +281,8 @@ bool ShaderFromNirProcessor::process_uniforms(nir_variable *uniform)
    auto type = uniform->type->is_array() ? uniform->type->without_array(): uniform->type;
    if (type->is_image() || uniform->data.mode == nir_var_mem_ssbo) {
       sh_info().uses_images = 1;
-      if (uniform->type->is_array())
-         sh_info().indirect_files |= TGSI_FILE_IMAGE;
+      if (uniform->type->is_array() && ! (uniform->data.mode == nir_var_mem_ssbo))
+         sh_info().indirect_files |= 1 << TGSI_FILE_IMAGE;
    }
 
    if (uniform->type->is_image()) {
@@ -285,26 +292,9 @@ bool ShaderFromNirProcessor::process_uniforms(nir_variable *uniform)
    return true;
 }
 
-bool ShaderFromNirProcessor::process_inputs(nir_variable *input)
+bool ShaderFromNirProcessor::scan_inputs_read(const nir_shader *sh)
 {
-   return do_process_inputs(input);
-}
-
-bool ShaderFromNirProcessor::process_outputs(nir_variable *output)
-{
-   return do_process_outputs(output);
-}
-
-void ShaderFromNirProcessor::add_array_deref(nir_deref_instr *instr)
-{
-   nir_variable *var = nir_deref_instr_get_variable(instr);
-
-   assert(nir_deref_mode_is(instr, nir_var_function_temp));
-   assert(glsl_type_is_array(var->type));
-
-   // add an alias for the index to the register(s);
-
-
+   return true;
 }
 
 void ShaderFromNirProcessor::set_var_address(nir_deref_instr *instr)
@@ -643,8 +633,6 @@ bool ShaderFromNirProcessor::emit_intrinsic_instruction(nir_intrinsic_instr* ins
          return false;
       }
       switch (mode_helper->second) {
-      case nir_var_shader_in:
-         return emit_load_input_deref(var, instr);
       case nir_var_function_temp:
          return emit_load_function_temp(var, instr);
       default:
@@ -657,8 +645,6 @@ bool ShaderFromNirProcessor::emit_intrinsic_instruction(nir_intrinsic_instr* ins
       return emit_store_scratch(instr);
    case nir_intrinsic_load_scratch:
       return emit_load_scratch(instr);
-   case nir_intrinsic_store_deref:
-      return emit_store_deref(instr);
    case nir_intrinsic_load_uniform:
       return load_uniform(instr);
    case nir_intrinsic_discard:
@@ -684,6 +670,8 @@ bool ShaderFromNirProcessor::emit_intrinsic_instruction(nir_intrinsic_instr* ins
    case nir_intrinsic_memory_barrier_image:
    case nir_intrinsic_group_memory_barrier:
       return emit_barrier(instr);
+   case nir_intrinsic_memory_barrier_atomic_counter:
+      return true;
    case nir_intrinsic_shared_atomic_add:
    case nir_intrinsic_shared_atomic_and:
    case nir_intrinsic_shared_atomic_or:
@@ -695,6 +683,8 @@ bool ShaderFromNirProcessor::emit_intrinsic_instruction(nir_intrinsic_instr* ins
    case nir_intrinsic_shared_atomic_exchange:
    case nir_intrinsic_shared_atomic_comp_swap:
       return emit_atomic_local_shared(instr);
+   case nir_intrinsic_shader_clock:
+      return emit_shader_clock(instr);
    case nir_intrinsic_copy_deref:
    case nir_intrinsic_load_constant:
    case nir_intrinsic_load_input:
@@ -780,6 +770,15 @@ bool ShaderFromNirProcessor::emit_load_scratch(nir_intrinsic_instr* instr)
    return true;
 }
 
+bool ShaderFromNirProcessor::emit_shader_clock(nir_intrinsic_instr* instr)
+{
+   emit_instruction(new AluInstruction(op1_mov, from_nir(instr->dest, 0),
+                                       PValue(new InlineConstValue(ALU_SRC_TIME_LO, 0)), EmitInstruction::write));
+   emit_instruction(new AluInstruction(op1_mov, from_nir(instr->dest, 1),
+                                       PValue(new InlineConstValue(ALU_SRC_TIME_HI, 0)), EmitInstruction::last_write));
+   return true;
+}
+
 GPRVector ShaderFromNirProcessor::vec_from_nir_with_fetch_constant(const nir_src& src,
                                                                    unsigned mask,
                                                                    const GPRVector::Swizzle& swizzle,
@@ -791,7 +790,7 @@ GPRVector ShaderFromNirProcessor::vec_from_nir_with_fetch_constant(const nir_src
    std::array<bool,4> used_swizzles = {false, false, false, false};
 
    /* Check whether all sources come from a GPR, and,
-    * if requested, whether they are swizzled as epected */
+    * if requested, whether they are swizzled as expected */
 
    for (int i = 0; i < 4 && use_same; ++i)  {
       if ((1 << i) & mask) {
@@ -847,7 +846,7 @@ GPRVector ShaderFromNirProcessor::vec_from_nir_with_fetch_constant(const nir_src
     */
    if (!use_same) {
       AluInstruction *ir = nullptr;
-      GPRVector result(allocate_temp_register(), swizzle);
+      GPRVector result = get_temp_vec4(swizzle);
       for (int i = 0; i < 4; ++i) {
          if (swizzle[i] < 4 && (mask & (1 << i))) {
             ir = new AluInstruction(op1_mov, result[i], from_nir(src, swizzle[i]),
@@ -887,7 +886,7 @@ bool ShaderFromNirProcessor::emit_load_ubo_vec4(nir_intrinsic_instr* instr)
       FetchInstruction *ir;
       if (bufid) {
          ir = new FetchInstruction(vc_fetch, no_index_offset, trgt, addr, 0,
-                                              1, nullptr, bim_none);
+                                              1 + bufid->u32, nullptr, bim_none);
       } else {
          PValue bufid = from_nir(instr->src[0], 0, 0);
          ir = new FetchInstruction(vc_fetch, no_index_offset, trgt, addr, 0,
@@ -956,12 +955,6 @@ bool ShaderFromNirProcessor::emit_discard_if(nir_intrinsic_instr* instr)
    return true;
 }
 
-bool ShaderFromNirProcessor::emit_load_input_deref(const nir_variable *var,
-                                                   nir_intrinsic_instr* instr)
-{
-   return do_emit_load_deref(var, instr);
-}
-
 bool ShaderFromNirProcessor::load_uniform(nir_intrinsic_instr* instr)
 {
    r600::sfn_log << SfnLog::instr << __func__ << ": emit '"
@@ -979,8 +972,7 @@ bool ShaderFromNirProcessor::load_uniform(nir_intrinsic_instr* instr)
 
    if (literal) {
       AluInstruction *ir = nullptr;
-
-      for (int i = 0; i < instr->num_components ; ++i) {
+      for (unsigned i = 0; i < nir_dest_num_components(instr->dest); ++i) {
          PValue u = PValue(new UniformValue(512 + literal->u32 + base, i));
          sfn_log << SfnLog::io << "uniform "
                  << instr->dest.ssa.index << " const["<< i << "]: "<< instr->const_index[i] << "\n";
@@ -1083,15 +1075,6 @@ PValue ShaderFromNirProcessor::from_nir_with_fetch_constant(const nir_src& src,
    return value;
 }
 
-bool ShaderFromNirProcessor::emit_store_deref(nir_intrinsic_instr* instr)
-{
-   auto out_var = get_deref_location(instr->src[0]);
-   if (!out_var)
-      return false;
-
-   return do_emit_store_deref(out_var, instr);
-}
-
 bool ShaderFromNirProcessor::emit_deref_instruction(nir_deref_instr* instr)
 {
    r600::sfn_log << SfnLog::instr << __func__ << ": emit '"
@@ -1166,6 +1149,20 @@ void ShaderFromNirProcessor::append_block(int nesting_change)
    m_output.push_back(InstructionBlock(m_nesting_depth, m_block_number++));
 }
 
+void ShaderFromNirProcessor::get_array_info(r600_shader& shader) const
+{
+   shader.num_arrays = m_reg_arrays.size();
+   if (shader.num_arrays) {
+      shader.arrays = (r600_shader_array *)calloc(shader.num_arrays, sizeof(r600_shader_array));
+      for (unsigned i = 0; i < shader.num_arrays; ++i) {
+         shader.arrays[i].comp_mask = m_reg_arrays[i]->mask();
+         shader.arrays[i].gpr_start = m_reg_arrays[i]->sel();
+         shader.arrays[i].gpr_count = m_reg_arrays[i]->size();
+      }
+      shader.indirect_files |= (1 << TGSI_FILE_TEMPORARY);
+   }
+}
+
 void ShaderFromNirProcessor::finalize()
 {
    do_finalize();
