@@ -82,6 +82,13 @@ struct dri_screen
    /* hooks filled in by dri2 & drisw */
    __DRIimage * (*lookup_egl_image)(struct dri_screen *ctx, void *handle);
 
+   /* DRI exts that vary based on gallium pipe_screen caps. */
+   __DRIimageExtension image_extension;
+   __DRI2bufferDamageExtension buffer_damage_extension;
+
+   /* DRI exts on this screen. Populated at init time based on device caps. */
+   const __DRIextension *screen_extensions[13];
+
    /* OpenCL interop */
    mtx_t opencl_func_mutex;
    opencl_dri_event_add_ref_t opencl_dri_event_add_ref;
@@ -118,6 +125,8 @@ struct __DRIimageRec {
    enum __DRIChromaSiting horizontal_siting;
    enum __DRIChromaSiting vertical_siting;
 
+   /* DRI loader screen */
+   __DRIscreen *sPriv;
 };
 
 static inline boolean
