@@ -1,5 +1,5 @@
 
-#include "state_tracker/graw.h"
+#include "frontend/graw.h"
 
 #include "pipe/p_context.h"
 #include "pipe/p_defines.h"
@@ -11,7 +11,7 @@
 #include "util/u_debug.h"
 #include "util/u_debug_image.h"
 #include "util/u_draw_quad.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_inlines.h"
 #include "util/u_memory.h"
 
@@ -43,6 +43,7 @@ graw_util_create_window(struct graw_info *info,
    int i;
 
    memset(info, 0, sizeof(*info));
+   memset(&resource_temp, 0, sizeof(resource_temp));
 
    /* It's hard to say whether window or screen should be created
     * first.  Different environments would prefer one or the other.
@@ -76,7 +77,6 @@ graw_util_create_window(struct graw_info *info,
       resource_temp.depth0 = 1;
       resource_temp.array_size = 1;
       resource_temp.last_level = 0;
-      resource_temp.nr_samples = 1;
       resource_temp.bind = (PIPE_BIND_RENDER_TARGET |
                             PIPE_BIND_DISPLAY_TARGET);
       info->color_buf[i] = info->screen->resource_create(info->screen,
@@ -108,7 +108,6 @@ graw_util_create_window(struct graw_info *info,
    resource_temp.depth0 = 1;
    resource_temp.array_size = 1;
    resource_temp.last_level = 0;
-   resource_temp.nr_samples = 1;
    resource_temp.bind = PIPE_BIND_DEPTH_STENCIL;
    info->zs_buf = info->screen->resource_create(info->screen, &resource_temp);
    if (!info->zs_buf) {
@@ -161,9 +160,9 @@ graw_util_default_state(struct graw_info *info, boolean depth_test)
       struct pipe_depth_stencil_alpha_state depthStencilAlpha;
       void *handle;
       memset(&depthStencilAlpha, 0, sizeof depthStencilAlpha);
-      depthStencilAlpha.depth.enabled = depth_test;
-      depthStencilAlpha.depth.writemask = 1;
-      depthStencilAlpha.depth.func = PIPE_FUNC_LESS;
+      depthStencilAlpha.depth_enabled = depth_test;
+      depthStencilAlpha.depth_writemask = 1;
+      depthStencilAlpha.depth_func = PIPE_FUNC_LESS;
       handle = info->ctx->create_depth_stencil_alpha_state(info->ctx,
                                                            &depthStencilAlpha);
       info->ctx->bind_depth_stencil_alpha_state(info->ctx, handle);
@@ -202,6 +201,11 @@ graw_util_viewport(struct graw_info *info,
    vp.translate[1] = half_height + y;
    vp.translate[2] = half_depth + z;
 
+   vp.swizzle_x = PIPE_VIEWPORT_SWIZZLE_POSITIVE_X;
+   vp.swizzle_y = PIPE_VIEWPORT_SWIZZLE_POSITIVE_Y;
+   vp.swizzle_z = PIPE_VIEWPORT_SWIZZLE_POSITIVE_Z;
+   vp.swizzle_w = PIPE_VIEWPORT_SWIZZLE_POSITIVE_W;
+
    info->ctx->set_viewport_states(info->ctx, 0, 1, &vp);
 }
 
@@ -209,7 +213,7 @@ graw_util_viewport(struct graw_info *info,
 static inline void
 graw_util_flush_front(const struct graw_info *info)
 {
-   info->screen->flush_frontbuffer(info->screen, info->color_buf[0],
+   info->screen->flush_frontbuffer(info->screen, info->ctx, info->color_buf[0],
                                    0, 0, info->window, NULL);
 }
 
@@ -224,6 +228,7 @@ graw_util_create_tex2d(const struct graw_info *info,
    struct pipe_resource temp, *tex;
    struct pipe_box box;
 
+   memset(&temp, 0, sizeof(temp));
    temp.target = PIPE_TEXTURE_2D;
    temp.format = format;
    temp.width0 = width;
@@ -231,7 +236,6 @@ graw_util_create_tex2d(const struct graw_info *info,
    temp.depth0 = 1;
    temp.last_level = 0;
    temp.array_size = 1;
-   temp.nr_samples = 1;
    temp.bind = PIPE_BIND_SAMPLER_VIEW;
    
    tex = info->screen->resource_create(info->screen, &temp);
@@ -242,14 +246,14 @@ graw_util_create_tex2d(const struct graw_info *info,
 
    u_box_2d(0, 0, width, height, &box);
 
-   info->ctx->transfer_inline_write(info->ctx,
-                                    tex,
-                                    0,
-                                    PIPE_TRANSFER_WRITE,
-                                    &box,
-                                    data,
-                                    row_stride,
-                                    image_bytes);
+   info->ctx->texture_subdata(info->ctx,
+                              tex,
+                              0,
+                              PIPE_MAP_WRITE,
+                              &box,
+                              data,
+                              row_stride,
+                              image_bytes);
 
    /* Possibly read back & compare against original data:
     */
@@ -259,7 +263,7 @@ graw_util_create_tex2d(const struct graw_info *info,
       uint32_t *ptr;
       t = pipe_transfer_map(info->ctx, samptex,
                             0, 0, /* level, layer */
-                            PIPE_TRANSFER_READ,
+                            PIPE_MAP_READ,
                             0, 0, SIZE, SIZE); /* x, y, width, height */
 
       ptr = info->ctx->transfer_map(info->ctx, t);
@@ -315,10 +319,10 @@ graw_util_create_simple_sampler_view(const struct graw_info *info,
    memset(&sv_temp, 0, sizeof(sv_temp));
    sv_temp.format = texture->format;
    sv_temp.texture = texture;
-   sv_temp.swizzle_r = PIPE_SWIZZLE_RED;
-   sv_temp.swizzle_g = PIPE_SWIZZLE_GREEN;
-   sv_temp.swizzle_b = PIPE_SWIZZLE_BLUE;
-   sv_temp.swizzle_a = PIPE_SWIZZLE_ALPHA;
+   sv_temp.swizzle_r = PIPE_SWIZZLE_X;
+   sv_temp.swizzle_g = PIPE_SWIZZLE_Y;
+   sv_temp.swizzle_b = PIPE_SWIZZLE_Z;
+   sv_temp.swizzle_a = PIPE_SWIZZLE_W;
 
    sv = info->ctx->create_sampler_view(info->ctx, texture, &sv_temp);
 
