@@ -24,6 +24,7 @@
 #include <pthread.h>
 
 #include "anv_private.h"
+#include "test_common.h"
 
 #define NUM_THREADS 8
 #define STATES_PER_THREAD_LOG2 12
@@ -31,16 +32,17 @@
 
 #include "state_pool_test_helper.h"
 
-int main(int argc, char **argv)
+int main(void)
 {
-   struct anv_instance instance;
+   struct anv_physical_device physical_device = { };
    struct anv_device device = {
-      .instance = &instance,
+      .physical = &physical_device,
    };
    struct anv_state_pool state_pool;
 
    pthread_mutex_init(&device.mutex, NULL);
-   anv_state_pool_init(&state_pool, &device, 4096);
+   anv_bo_cache_init(&device.bo_cache);
+   anv_state_pool_init(&state_pool, &device, "test", 4096, 0, 4096);
 
    /* Grab one so a zero offset is impossible */
    anv_state_pool_alloc(&state_pool, 16, 16);
@@ -52,7 +54,7 @@ int main(int argc, char **argv)
       struct anv_state states[NUM_THREADS * STATES_PER_THREAD];
       for (unsigned i = 0; i < NUM_THREADS * STATES_PER_THREAD; i++) {
          states[i] = anv_state_pool_alloc(&state_pool, 16, 16);
-         assert(states[i].offset != 0);
+         ASSERT(states[i].offset != 0);
       }
 
       for (unsigned i = 0; i < NUM_THREADS * STATES_PER_THREAD; i++)
