@@ -114,7 +114,7 @@ st_renderbuffer_alloc_storage(struct gl_context * ctx,
                               GLuint width, GLuint height)
 {
    struct st_context *st = st_context(ctx);
-   struct pipe_screen *screen = st->pipe->screen;
+   struct pipe_screen *screen = st->screen;
    struct st_renderbuffer *strb = st_renderbuffer(rb);
    enum pipe_format format = PIPE_FORMAT_NONE;
    struct pipe_resource templ;
@@ -447,30 +447,6 @@ st_new_renderbuffer_fb(enum pipe_format format, unsigned samples, boolean sw)
    return &strb->Base;
 }
 
-void
-st_regen_renderbuffer_surface(struct st_context *st,
-                              struct st_renderbuffer *strb)
-{
-   struct pipe_context *pipe = st->pipe;
-   struct pipe_resource *resource = strb->texture;
-
-   struct pipe_surface **psurf =
-      strb->surface_srgb ? &strb->surface_srgb : &strb->surface_linear;
-   struct pipe_surface *surf = *psurf;
-   /* create a new pipe_surface */
-   struct pipe_surface surf_tmpl;
-   memset(&surf_tmpl, 0, sizeof(surf_tmpl));
-   surf_tmpl.format = surf->format;
-   surf_tmpl.nr_samples = strb->rtt_nr_samples;
-   surf_tmpl.u.tex.level = surf->u.tex.level;
-   surf_tmpl.u.tex.first_layer = surf->u.tex.first_layer;
-   surf_tmpl.u.tex.last_layer = surf->u.tex.last_layer;
-
-   pipe_surface_release(pipe, psurf);
-
-   *psurf = pipe->create_surface(pipe, resource, &surf_tmpl);
-   strb->surface = *psurf;
-}
 
 /**
  * Create or update the pipe_surface of a FBO renderbuffer.
@@ -537,11 +513,12 @@ st_update_renderbuffer_surface(struct st_context *st,
    if (strb->is_rtt && resource->array_size > 1 &&
        stTexObj->base.Immutable) {
       const struct gl_texture_object *tex = &stTexObj->base;
-      first_layer += tex->MinLayer;
+      first_layer += tex->Attrib.MinLayer;
       if (!strb->rtt_layered)
-         last_layer += tex->MinLayer;
+         last_layer += tex->Attrib.MinLayer;
       else
-         last_layer = MIN2(first_layer + tex->NumLayers - 1, last_layer);
+         last_layer = MIN2(first_layer + tex->Attrib.NumLayers - 1,
+                           last_layer);
    }
 
    struct pipe_surface **psurf =
@@ -726,7 +703,7 @@ static void
 st_validate_framebuffer(struct gl_context *ctx, struct gl_framebuffer *fb)
 {
    struct st_context *st = st_context(ctx);
-   struct pipe_screen *screen = st->pipe->screen;
+   struct pipe_screen *screen = st->screen;
    const struct gl_renderbuffer_attachment *depth =
          &fb->Attachment[BUFFER_DEPTH];
    const struct gl_renderbuffer_attachment *stencil =
